# This file is part of Tryton.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.

from trytond.pool import Pool
from . import caldav
from .webdav import *
from .calendar_ import *
from .res import *


def register():
    Pool.register(
        Collection,
        Calendar,
        ReadUser,
        WriteUser,
        Category,
        Location,
        Event,
        EventCategory,
        EventAlarm,
        EventAttendee,
        EventRDate,
        EventExDate,
        EventRRule,
        EventExRule,
        User,
        module='calendar', type_='model')



#Copyright (c) 2005 Simon Pamies (s.pamies@banality.de)
#Copyright (c) 2003 Clark Evans
#Copyright (c) 2002 Noah Spurrier
#Copyright (c) 2001 Juergen Hermann
#
#This library is free software; you can redistribute it and/or
#modify it under the terms of the GNU Library General Public
#License as published by the Free Software Foundation; either
#version 2 of the License, or (at your option) any later version.
#
#This library is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#Library General Public License for more details.
#
#You should have received a copy of the GNU Library General Public
#License along with this library; if not, write to the Free
#Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
#MA 02111-1307, USA


'''
    This module is used to fork the current process into a daemon.
    Almost none of this is necessary (or advisable) if your daemon
    is being started by inetd. In that case, stdin, stdout and stderr are
    all set up for you to refer to the network connection, and the fork()s
    and session manipulation should not be done (to avoid confusing inetd).
    Only the chdir() and umask() steps remain as useful.
    References:
        UNIX Programming FAQ
            1.7 How do I get my program to act like a daemon?
                http://www.erlenstar.demon.co.uk/unix/faq_2.html#SEC16
        Advanced Programming in the Unix Environment
            W. Richard Stevens, 1992, Addison-Wesley, ISBN 0-201-56317-7.

    History:
      2005/06/23 by Simon Pamies
      2001/07/10 by Juergen Hermann
      2002/08/28 by Noah Spurrier
      2003/02/24 by Clark Evans
      
      http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/66012
'''


import sys, os, time
from signal import SIGTERM

def deamonize(stdout='/dev/null', stderr=None, stdin='/dev/null',
              pidfile=None, startmsg = 'started with pid %s' ):
    '''
        This forks the current process into a daemon.
        The stdin, stdout, and stderr arguments are file names that
        will be opened and be used to replace the standard file descriptors
        in sys.stdin, sys.stdout, and sys.stderr.
        These arguments are optional and default to /dev/null.
        Note that stderr is opened unbuffered, so
        if it shares a file with stdout then interleaved output
        may not appear in the order that you expect.
    '''
    # Do first fork.
    try:
        pid = os.fork()
        if pid > 0: sys.exit(0) # Exit first parent.
    except OSError as e:
        sys.stderr.write("fork #1 failed: (%d) %s\n" % (e.errno, e.strerror))
        sys.exit(1)

    # Decouple from parent environment.
    os.chdir("/")
    os.umask(0)
    os.setsid()

    # Do second fork.
    try:
        pid = os.fork()
        if pid > 0: sys.exit(0) # Exit second parent.
    except OSError as e:
        sys.stderr.write("fork #2 failed: (%d) %s\n" % (e.errno, e.strerror))
        sys.exit(1)

    # Open file descriptors and print start message
    if not stderr: stderr = stdout
    si = open(stdin, 'r')
    so = open(stdout, 'a+')
    se = open(stderr, 'a+', 0)
    pid = str(os.getpid())
    sys.stderr.write("\n%s\n" % startmsg % pid)
    sys.stderr.flush()
    if pidfile: open(pidfile,'w+').write("%s\n" % pid)

    if sys.stdin.closed: sys.stdin = open('/dev/null', 'r')
    if sys.stdout.closed: sys.stdout = open('/dev/null', 'a+')
    if sys.stderr.closed: sys.stderr = open('/dev/null', 'a+')

    # Redirect standard file descriptors.
    os.dup2(si.fileno(), sys.stdin.fileno())
    os.dup2(so.fileno(), sys.stdout.fileno())
    os.dup2(se.fileno(), sys.stderr.fileno())

def startstop(stdout='/dev/null', stderr=None, stdin='/dev/null',
              pidfile='pid.txt', startmsg = 'started with pid %s', action='start' ):
    if action:
        try:
            pf  = open(pidfile,'r')
            pid = int(pf.read().strip())
            pf.close()
        except IOError:
            pid = None

        if 'stop' == action or 'restart' == action:
            if not pid:
                mess = "Could not stop, pid file '%s' missing.\n"
                sys.stderr.write(mess % pidfile)
                if 'stop' == action:
                    sys.exit(1)
                action = 'start'
                pid = None
            else:
               try:
                  while 1:
                      os.kill(pid,SIGTERM)
                      time.sleep(1)
               except OSError as err:
                  err = str(err)
                  if err.find("No such process") > 0:
                      os.remove(pidfile)
                      if 'stop' == action:
                          sys.exit(0)
                      action = 'start'
                      pid = None
                  else:
                      print((str(err)))
                      sys.exit(1)

        if 'start' == action:
            if pid:
                mess = "Start aborted since pid file '%s' exists.\n"
                sys.stderr.write(mess % pidfile)
                sys.exit(1)

            deamonize(stdout,stderr,stdin,pidfile,startmsg)
            return

        if 'status' == action:
            if not pid:
                sys.stderr.write('Status: Stopped\n')

            else: sys.stderr.write('Status: Running\n')
            sys.exit(0)




##############################################################################
#
#    GNU Health: Reporting Module
#
#
#    Copyright (C) 2012-2014  Sebastian Marro <smarro@gnusolidario.org>
#    Copyright (C) 2013-2017 Luis Falcon <lfalcon@gnusolidario.org>
#    Copyright (C) 2011-2017 GNU Solidario <health@gnusolidario.org>
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from sql import Literal, Join
from sql.aggregate import Max, Count
from trytond.model import ModelView, ModelSQL, fields
from trytond.wizard import Wizard, StateView, StateAction, StateTransition, \
    Button
from trytond.pyson import PYSONEncoder
from trytond.pool import Pool
from trytond.transaction import Transaction


__all__ = ['OpenEvaluationsStart', 'OpenEvaluations', 'EvaluationsDoctor',
    'EvaluationsSpecialty', 'EvaluationsSector']


class OpenEvaluationsStart(ModelView):
    'Open Evaluations'
    __name__ = 'gnuhealth.evaluations.open.start'

    start_date = fields.Date('Start Date')
    end_date = fields.Date('End Date')
    group_by = fields.Selection([
        ('doctor', 'Doctor'),
        ('specialty', 'Specialty'),
        ('sector', 'Sector'),
        ], 'Group By', sort=False, required=True)


class OpenEvaluations(Wizard):
    'Open Evaluations'
    __name__ = 'gnuhealth.evaluations.open'

    start = StateView('gnuhealth.evaluations.open.start',
        'health_reporting.evaluations_open_start_view_form', [
            Button('Cancel', 'end', 'tryton-cancel'),
            Button('Open', 'select', 'tryton-ok', default=True),
            ])
    select = StateTransition()
    open_doctor = StateAction('health_reporting.act_evaluations_doctor')
    open_specialty = StateAction('health_reporting.act_evaluations_specialty')
    open_sector = StateAction('health_reporting.act_evaluations_sector')

    def transition_select(self):
        return 'open_' + self.start.group_by

    def do_open_doctor(self, action):
        action['pyson_context'] = PYSONEncoder().encode({
                'start_date': self.start.start_date,
                'end_date': self.start.end_date,
                })
        return action, {}

    def do_open_specialty(self, action):
        action['pyson_context'] = PYSONEncoder().encode({
                'start_date': self.start.start_date,
                'end_date': self.start.end_date,
                })
        return action, {}

    def do_open_sector(self, action):
        action['pyson_context'] = PYSONEncoder().encode({
                'start_date': self.start.start_date,
                'end_date': self.start.end_date,
                })
        return action, {}

    def transition_open_doctor(self):
        return 'end'

    def transition_open_specialty(self):
        return 'end'

    def transition_open_sector(self):
        return 'end'


class EvaluationsDoctor(ModelSQL, ModelView):
    'Evaluations per Doctor'
    __name__ = 'gnuhealth.evaluations_doctor'

    doctor = fields.Many2One('gnuhealth.healthprofessional', 'Doctor')
    evaluations = fields.Integer('Evaluations')

    @staticmethod
    def table_query():
        pool = Pool()
        Evaluation = pool.get('gnuhealth.patient.evaluation')
        evaluation = Evaluation.__table__()
        where = Literal(True)
        if Transaction().context.get('start_date'):
            where &= evaluation.evaluation_start >= \
                Transaction().context['start_date']
        if Transaction().context.get('end_date'):
            where &= evaluation.evaluation_start <= \
                Transaction().context['end_date']

        return evaluation.select(
            evaluation.healthprof.as_('id'),
            Max(evaluation.create_uid).as_('create_uid'),
            Max(evaluation.create_date).as_('create_date'),
            Max(evaluation.write_uid).as_('write_uid'),
            Max(evaluation.write_date).as_('write_date'),
            evaluation.healthprof.as_('doctor'),
            Count(evaluation.id).as_('evaluations'),
            where=where,
            group_by=evaluation.healthprof)


class EvaluationsSpecialty(ModelSQL, ModelView):
    'Evaluations per Specialty'
    __name__ = 'gnuhealth.evaluations_specialty'

    specialty = fields.Many2One('gnuhealth.specialty', 'Specialty')
    evaluations = fields.Integer('Evaluations')

    @staticmethod
    def table_query():
        pool = Pool()
        Evaluation = pool.get('gnuhealth.patient.evaluation')
        evaluation = Evaluation.__table__()
        where = evaluation.specialty != None
        if Transaction().context.get('start_date'):
            where &= evaluation.evaluation_start >= \
                Transaction().context['start_date']
        if Transaction().context.get('end_date'):
            where &= evaluation.evaluation_start <= \
                Transaction().context['end_date']

        return evaluation.select(
            evaluation.specialty.as_('id'),
            Max(evaluation.create_uid).as_('create_uid'),
            Max(evaluation.create_date).as_('create_date'),
            Max(evaluation.write_uid).as_('write_uid'),
            Max(evaluation.write_date).as_('write_date'),
            evaluation.specialty,
            Count(evaluation.specialty).as_('evaluations'),
            where=where,
            group_by=evaluation.specialty)


class EvaluationsSector(ModelSQL, ModelView):
    'Evaluations per Sector'
    __name__ = 'gnuhealth.evaluations_sector'

    sector = fields.Many2One('gnuhealth.operational_sector', 'Sector')
    evaluations = fields.Integer('Evaluations')

    @staticmethod
    def table_query():
        pool = Pool()
        evaluation = pool.get('gnuhealth.patient.evaluation').__table__()
        party = pool.get('party.party').__table__()
        patient = pool.get('gnuhealth.patient').__table__()
        du = pool.get('gnuhealth.du').__table__()
        sector = pool.get('gnuhealth.operational_sector').__table__()
        join1 = Join(evaluation, patient)
        join1.condition = join1.right.id == evaluation.patient
        join2 = Join(join1, party)
        join2.condition = join2.right.id == join1.right.name
        join3 = Join(join2, du)
        join3.condition = join3.right.id == join2.right.du
        join4 = Join(join3, sector)
        join4.condition = join4.right.id == join3.right.operational_sector
        where = Literal(True)
        if Transaction().context.get('start_date'):
            where &= evaluation.evaluation_start >= \
                Transaction().context['start_date']
        if Transaction().context.get('end_date'):
            where &= evaluation.evaluation_start <= \
                Transaction().context['end_date']

        return join4.select(
            join4.right.id,
            Max(evaluation.create_uid).as_('create_uid'),
            Max(evaluation.create_date).as_('create_date'),
            Max(evaluation.write_uid).as_('write_uid'),
            Max(evaluation.write_date).as_('write_date'),
            join4.right.id.as_('sector'),
            Count(join4.right.id).as_('evaluations'),
            where=where,
            group_by=join4.right.id)




import unittest
import trytond.tests.test_tryton
from trytond.tests.test_tryton import ModuleTestCase


class HealthPediatricsGrowthChartsTestCase(ModuleTestCase):
    '''
    Test Health Pediatrics Growth Charts module.
    '''
    module = 'health_pediatrics_growth_charts'


def suite():
    suite = trytond.tests.test_tryton.suite()
    suite.addTests(unittest.TestLoader().loadTestsFromTestCase(
        HealthPediatricsGrowthChartsTestCase))
    return suite



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2014 Sebastián Marro <smarro@thymbra.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from datetime import timedelta, datetime, time
import pytz
from trytond.model import ModelView, fields
from trytond.wizard import Wizard, StateView, StateAction, StateTransition, \
    Button
from trytond.pyson import PYSONEncoder
from trytond.pool import Pool
from trytond.transaction import Transaction

__all__ = ['CreateAppointmentStart', 'CreateAppointment']


class CreateAppointmentStart(ModelView):
    'Create Appointments Start'
    __name__ = 'gnuhealth.calendar.create.appointment.start'

    healthprof = fields.Many2One('gnuhealth.healthprofessional', 'Health Prof',
        required=True)
    specialty = fields.Many2One('gnuhealth.specialty', 'Specialty',
        required=True)
    institution = fields.Many2One('gnuhealth.institution', 'Institution',
        required=True)
    date_start = fields.Date('Start Date', required=True)
    date_end = fields.Date('End Date', required=True)
    time_start = fields.Time('Start Time', required=True, format='%H:%M')
    time_end = fields.Time('End Time', required=True, format='%H:%M')
    appointment_minutes = fields.Integer('Appointment Minutes', required=True)
    monday = fields.Boolean('Monday')
    tuesday = fields.Boolean('Tuesday')
    wednesday = fields.Boolean('Wednesday')
    thursday = fields.Boolean('Thursday')
    friday = fields.Boolean('Friday')
    saturday = fields.Boolean('Saturday')
    sunday = fields.Boolean('Sunday')

    @staticmethod
    def default_institution():
        HealthInst = Pool().get('gnuhealth.institution')
        return HealthInst.get_institution()

    @fields.depends('healthprof')
    def on_change_with_specialty(self):
        # Return the Current / Main speciality of the Health Professional
        # if this speciality has been specified in the HP record.
        if (self.healthprof and self.healthprof.main_specialty):
            specialty = self.healthprof.main_specialty.specialty.id
            return specialty


class CreateAppointment(Wizard):
    'Create Appointment'
    __name__ = 'gnuhealth.calendar.create.appointment'

    start = StateView('gnuhealth.calendar.create.appointment.start',
        'health_calendar.create_appointment_start_view_form', [
            Button('Cancel', 'end', 'tryton-cancel'),
            Button('Create', 'create_', 'tryton-ok', default=True),
            ])
    create_ = StateTransition()
    open_ = StateAction('health.action_gnuhealth_appointment_view')

    @classmethod
    def __setup__(cls):
        super(CreateAppointment, cls).__setup__()
        cls._error_messages.update({
            'no_company_timezone':
                'You need to define a timezone for this company'
                ' before creating a work schedule.\n\n'
                'Party > Configuration > Companies',
            'end_before_start': 'End date before start',
            'period_too_long': 'The schedule period should be < 31 days',
        })

    def transition_create_(self):
        pool = Pool()
        Appointment = pool.get('gnuhealth.appointment')
        Company = pool.get('company.company')

        timezone = None
        company_id = Transaction().context.get('company')
        if company_id:
            company = Company(company_id)
            if company.timezone:
                timezone = pytz.timezone(company.timezone)
            else:
                self.raise_user_error('no_company_timezone')

        appointments = []

        # Iterate over days
        day_count = (self.start.date_end - self.start.date_start).days + 1
        
        # Validate dates
        if (self.start.date_start and self.start.date_end):
            if (self.start.date_end < self.start.date_start):
                self.raise_user_error('end_before_start')

            if (day_count > 31):
                self.raise_user_error('period_too_long')
        
        for single_date in (self.start.date_start + timedelta(n)
            for n in range(day_count)):
            if ((single_date.weekday() == 0 and self.start.monday)
            or (single_date.weekday() == 1 and self.start.tuesday)
            or (single_date.weekday() == 2 and self.start.wednesday)
            or (single_date.weekday() == 3 and self.start.thursday)
            or (single_date.weekday() == 4 and self.start.friday)
            or (single_date.weekday() == 5 and self.start.saturday)
            or (single_date.weekday() == 6 and self.start.sunday)):
                # Iterate over time
                dt = datetime.combine(
                    single_date, self.start.time_start)
                dt = timezone.localize(dt)
                dt = dt.astimezone(pytz.utc) 
                dt_end = datetime.combine(
                    single_date, self.start.time_end)
                dt_end = timezone.localize(dt_end)
                dt_end = dt_end.astimezone(pytz.utc) 
                while dt < dt_end:
                    appointment = {
                        'healthprof': self.start.healthprof.id,
                        'speciality': self.start.specialty.id,
                        'institution': self.start.institution.id,
                        'appointment_date': dt,
                        'appointment_date_end': dt +
                            timedelta(minutes=self.start.appointment_minutes),
                        'state': 'free',
                        }
                    appointments.append(appointment)
                    dt += timedelta(minutes=self.start.appointment_minutes)
        if appointments:
            Appointment.create(appointments)
        return 'open_'

    def do_open_(self, action):
        action['pyson_domain'] = [
            ('healthprof', '=', self.start.healthprof.id),
            ('appointment_date', '>=',
                datetime.combine(self.start.date_start, time())),
            ('appointment_date', '<=',
                datetime.combine(self.start.date_end, time())),
            ]
        action['pyson_domain'] = PYSONEncoder().encode(action['pyson_domain'])
        action['name'] += ' - %s, %s' % (self.start.healthprof.name.lastname,
                                         self.start.healthprof.name.name)
        return action, {}

    def transition_open_(self):
        return 'end'



from .base_classes import *
from .support_functions import *


class DiagnosticReport(Resource):
    """The findings and interpretation of diagnostic tests performed on
    patients, groups of patients, devices, and locations, and/or
    specimens derived from these. The report includes clinical
    context such as requesting and provider information, and some
    mix of atomic results, images, textual and coded interpretation,
    and formatted representation of diagnostic reports.If the
    element is present, it must have either a @value, an @id, or
    extensionsThe time or time-period the observed values are
    related to. This is usually either the time of the procedure or
    of specimen collection(s), but very often the source of the
    date/time is not known, only the date/time itself."""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, name=None, status=None, issued=None, subject=None, performer=None, identifier=None, requestDetail=None, serviceCategory=None, diagnosticDateTime=None, diagnosticPeriod=None, specimen=None, result=None, imagingStudy=None, image=None, conclusion=None, codedDiagnosis=None, presentedForm=None):
        self.original_tagname_ = None
        super(DiagnosticReport, self).__init__(id, extension, modifierExtension, language, text, contained, )
        self.name = name
        self.status = status
        self.issued = issued
        self.subject = subject
        self.performer = performer
        self.identifier = identifier
        if requestDetail is None:
            self.requestDetail = []
        else:
            self.requestDetail = requestDetail
        self.serviceCategory = serviceCategory
        self.diagnosticDateTime = diagnosticDateTime
        self.diagnosticPeriod = diagnosticPeriod
        if specimen is None:
            self.specimen = []
        else:
            self.specimen = specimen
        if result is None:
            self.result = []
        else:
            self.result = result
        if imagingStudy is None:
            self.imagingStudy = []
        else:
            self.imagingStudy = imagingStudy
        if image is None:
            self.image = []
        else:
            self.image = image
        self.conclusion = conclusion
        if codedDiagnosis is None:
            self.codedDiagnosis = []
        else:
            self.codedDiagnosis = codedDiagnosis
        if presentedForm is None:
            self.presentedForm = []
        else:
            self.presentedForm = presentedForm
    def factory(*args_, **kwargs_):
        if DiagnosticReport.subclass:
            return DiagnosticReport.subclass(*args_, **kwargs_)
        else:
            return DiagnosticReport(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_name(self): return self.name
    def set_name(self, name): self.name = name
    def get_status(self): return self.status
    def set_status(self, status): self.status = status
    def get_issued(self): return self.issued
    def set_issued(self, issued): self.issued = issued
    def get_subject(self): return self.subject
    def set_subject(self, subject): self.subject = subject
    def get_performer(self): return self.performer
    def set_performer(self, performer): self.performer = performer
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def get_requestDetail(self): return self.requestDetail
    def set_requestDetail(self, requestDetail): self.requestDetail = requestDetail
    def add_requestDetail(self, value): self.requestDetail.append(value)
    def insert_requestDetail(self, index, value): self.requestDetail[index] = value
    def get_serviceCategory(self): return self.serviceCategory
    def set_serviceCategory(self, serviceCategory): self.serviceCategory = serviceCategory
    def get_diagnosticDateTime(self): return self.diagnosticDateTime
    def set_diagnosticDateTime(self, diagnosticDateTime): self.diagnosticDateTime = diagnosticDateTime
    def get_diagnosticPeriod(self): return self.diagnosticPeriod
    def set_diagnosticPeriod(self, diagnosticPeriod): self.diagnosticPeriod = diagnosticPeriod
    def get_specimen(self): return self.specimen
    def set_specimen(self, specimen): self.specimen = specimen
    def add_specimen(self, value): self.specimen.append(value)
    def insert_specimen(self, index, value): self.specimen[index] = value
    def get_result(self): return self.result
    def set_result(self, result): self.result = result
    def add_result(self, value): self.result.append(value)
    def insert_result(self, index, value): self.result[index] = value
    def get_imagingStudy(self): return self.imagingStudy
    def set_imagingStudy(self, imagingStudy): self.imagingStudy = imagingStudy
    def add_imagingStudy(self, value): self.imagingStudy.append(value)
    def insert_imagingStudy(self, index, value): self.imagingStudy[index] = value
    def get_image(self): return self.image
    def set_image(self, image): self.image = image
    def add_image(self, value): self.image.append(value)
    def insert_image(self, index, value): self.image[index] = value
    def get_conclusion(self): return self.conclusion
    def set_conclusion(self, conclusion): self.conclusion = conclusion
    def get_codedDiagnosis(self): return self.codedDiagnosis
    def set_codedDiagnosis(self, codedDiagnosis): self.codedDiagnosis = codedDiagnosis
    def add_codedDiagnosis(self, value): self.codedDiagnosis.append(value)
    def insert_codedDiagnosis(self, index, value): self.codedDiagnosis[index] = value
    def get_presentedForm(self): return self.presentedForm
    def set_presentedForm(self, presentedForm): self.presentedForm = presentedForm
    def add_presentedForm(self, value): self.presentedForm.append(value)
    def insert_presentedForm(self, index, value): self.presentedForm[index] = value
    def hasContent_(self):
        if (
            self.name is not None or
            self.status is not None or
            self.issued is not None or
            self.subject is not None or
            self.performer is not None or
            self.identifier is not None or
            self.requestDetail or
            self.serviceCategory is not None or
            self.diagnosticDateTime is not None or
            self.diagnosticPeriod is not None or
            self.specimen or
            self.result or
            self.imagingStudy or
            self.image or
            self.conclusion is not None or
            self.codedDiagnosis or
            self.presentedForm or
            super(DiagnosticReport, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='DiagnosticReport', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='DiagnosticReport')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='DiagnosticReport', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='DiagnosticReport'):
        super(DiagnosticReport, self).exportAttributes(outfile, level, already_processed, namespace_, name_='DiagnosticReport')
    def exportChildren(self, outfile, level, namespace_='', name_='DiagnosticReport', fromsubclass_=False, pretty_print=True):
        super(DiagnosticReport, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.name is not None:
            self.name.export(outfile, level, namespace_, name_='name', pretty_print=pretty_print)
        if self.status is not None:
            self.status.export(outfile, level, namespace_, name_='status', pretty_print=pretty_print)
        if self.issued is not None:
            self.issued.export(outfile, level, namespace_, name_='issued', pretty_print=pretty_print)
        if self.subject is not None:
            self.subject.export(outfile, level, namespace_, name_='subject', pretty_print=pretty_print)
        if self.performer is not None:
            self.performer.export(outfile, level, namespace_, name_='performer', pretty_print=pretty_print)
        if self.identifier is not None:
            self.identifier.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        for requestDetail_ in self.requestDetail:
            requestDetail_.export(outfile, level, namespace_, name_='requestDetail', pretty_print=pretty_print)
        if self.serviceCategory is not None:
            self.serviceCategory.export(outfile, level, namespace_, name_='serviceCategory', pretty_print=pretty_print)
        if self.diagnosticDateTime is not None:
            self.diagnosticDateTime.export(outfile, level, namespace_, name_='diagnosticDateTime', pretty_print=pretty_print)
        if self.diagnosticPeriod is not None:
            self.diagnosticPeriod.export(outfile, level, namespace_, name_='diagnosticPeriod', pretty_print=pretty_print)
        for specimen_ in self.specimen:
            specimen_.export(outfile, level, namespace_, name_='specimen', pretty_print=pretty_print)
        for result_ in self.result:
            result_.export(outfile, level, namespace_, name_='result', pretty_print=pretty_print)
        for imagingStudy_ in self.imagingStudy:
            imagingStudy_.export(outfile, level, namespace_, name_='imagingStudy', pretty_print=pretty_print)
        for image_ in self.image:
            image_.export(outfile, level, namespace_, name_='image', pretty_print=pretty_print)
        if self.conclusion is not None:
            self.conclusion.export(outfile, level, namespace_, name_='conclusion', pretty_print=pretty_print)
        for codedDiagnosis_ in self.codedDiagnosis:
            codedDiagnosis_.export(outfile, level, namespace_, name_='codedDiagnosis', pretty_print=pretty_print)
        for presentedForm_ in self.presentedForm:
            presentedForm_.export(outfile, level, namespace_, name_='presentedForm', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='DiagnosticReport', mapping_=None):
        element = super(DiagnosticReport, self).to_etree(parent_element, name_, mapping_)
        if self.name is not None:
            name_ = self.name
            name_.to_etree(element, name_='name', mapping_=mapping_)
        if self.status is not None:
            status_ = self.status
            status_.to_etree(element, name_='status', mapping_=mapping_)
        if self.issued is not None:
            issued_ = self.issued
            issued_.to_etree(element, name_='issued', mapping_=mapping_)
        if self.subject is not None:
            subject_ = self.subject
            subject_.to_etree(element, name_='subject', mapping_=mapping_)
        if self.performer is not None:
            performer_ = self.performer
            performer_.to_etree(element, name_='performer', mapping_=mapping_)
        if self.identifier is not None:
            identifier_ = self.identifier
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        for requestDetail_ in self.requestDetail:
            requestDetail_.to_etree(element, name_='requestDetail', mapping_=mapping_)
        if self.serviceCategory is not None:
            serviceCategory_ = self.serviceCategory
            serviceCategory_.to_etree(element, name_='serviceCategory', mapping_=mapping_)
        if self.diagnosticDateTime is not None:
            diagnosticDateTime_ = self.diagnosticDateTime
            diagnosticDateTime_.to_etree(element, name_='diagnosticDateTime', mapping_=mapping_)
        if self.diagnosticPeriod is not None:
            diagnosticPeriod_ = self.diagnosticPeriod
            diagnosticPeriod_.to_etree(element, name_='diagnosticPeriod', mapping_=mapping_)
        for specimen_ in self.specimen:
            specimen_.to_etree(element, name_='specimen', mapping_=mapping_)
        for result_ in self.result:
            result_.to_etree(element, name_='result', mapping_=mapping_)
        for imagingStudy_ in self.imagingStudy:
            imagingStudy_.to_etree(element, name_='imagingStudy', mapping_=mapping_)
        for image_ in self.image:
            image_.to_etree(element, name_='image', mapping_=mapping_)
        if self.conclusion is not None:
            conclusion_ = self.conclusion
            conclusion_.to_etree(element, name_='conclusion', mapping_=mapping_)
        for codedDiagnosis_ in self.codedDiagnosis:
            codedDiagnosis_.to_etree(element, name_='codedDiagnosis', mapping_=mapping_)
        for presentedForm_ in self.presentedForm:
            presentedForm_.to_etree(element, name_='presentedForm', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(DiagnosticReport, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'name':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.name = obj_
            obj_.original_tagname_ = 'name'
        elif nodeName_ == 'status':
            obj_ = DiagnosticReportStatus.factory()
            obj_.build(child_)
            self.status = obj_
            obj_.original_tagname_ = 'status'
        elif nodeName_ == 'issued':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.issued = obj_
            obj_.original_tagname_ = 'issued'
        elif nodeName_ == 'subject':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.subject = obj_
            obj_.original_tagname_ = 'subject'
        elif nodeName_ == 'performer':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.performer = obj_
            obj_.original_tagname_ = 'performer'
        elif nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier = obj_
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'requestDetail':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.requestDetail.append(obj_)
            obj_.original_tagname_ = 'requestDetail'
        elif nodeName_ == 'serviceCategory':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.serviceCategory = obj_
            obj_.original_tagname_ = 'serviceCategory'
        elif nodeName_ == 'diagnosticDateTime':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.diagnosticDateTime = obj_
            obj_.original_tagname_ = 'diagnosticDateTime'
        elif nodeName_ == 'diagnosticPeriod':
            obj_ = Period.factory()
            obj_.build(child_)
            self.diagnosticPeriod = obj_
            obj_.original_tagname_ = 'diagnosticPeriod'
        elif nodeName_ == 'specimen':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.specimen.append(obj_)
            obj_.original_tagname_ = 'specimen'
        elif nodeName_ == 'result':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.result.append(obj_)
            obj_.original_tagname_ = 'result'
        elif nodeName_ == 'imagingStudy':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.imagingStudy.append(obj_)
            obj_.original_tagname_ = 'imagingStudy'
        elif nodeName_ == 'image':
            obj_ = DiagnosticReport_Image.factory()
            obj_.build(child_)
            self.image.append(obj_)
            obj_.original_tagname_ = 'image'
        elif nodeName_ == 'conclusion':
            obj_ = string.factory()
            obj_.build(child_)
            self.conclusion = obj_
            obj_.original_tagname_ = 'conclusion'
        elif nodeName_ == 'codedDiagnosis':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.codedDiagnosis.append(obj_)
            obj_.original_tagname_ = 'codedDiagnosis'
        elif nodeName_ == 'presentedForm':
            obj_ = Attachment.factory()
            obj_.build(child_)
            self.presentedForm.append(obj_)
            obj_.original_tagname_ = 'presentedForm'
        super(DiagnosticReport, self).buildChildren(child_, node, nodeName_, True)
# end class DiagnosticReport


class DiagnosticReport_Image(BackboneElement):
    """The findings and interpretation of diagnostic tests performed on
    patients, groups of patients, devices, and locations, and/or
    specimens derived from these. The report includes clinical
    context such as requesting and provider information, and some
    mix of atomic results, images, textual and coded interpretation,
    and formatted representation of diagnostic reports."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, comment=None, link=None):
        self.original_tagname_ = None
        super(DiagnosticReport_Image, self).__init__(id, extension, modifierExtension, )
        self.comment = comment
        self.link = link
    def factory(*args_, **kwargs_):
        if DiagnosticReport_Image.subclass:
            return DiagnosticReport_Image.subclass(*args_, **kwargs_)
        else:
            return DiagnosticReport_Image(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_comment(self): return self.comment
    def set_comment(self, comment): self.comment = comment
    def get_link(self): return self.link
    def set_link(self, link): self.link = link
    def hasContent_(self):
        if (
            self.comment is not None or
            self.link is not None or
            super(DiagnosticReport_Image, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='DiagnosticReport.Image', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='DiagnosticReport.Image')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='DiagnosticReport.Image', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='DiagnosticReport.Image'):
        super(DiagnosticReport_Image, self).exportAttributes(outfile, level, already_processed, namespace_, name_='DiagnosticReport.Image')
    def exportChildren(self, outfile, level, namespace_='', name_='DiagnosticReport.Image', fromsubclass_=False, pretty_print=True):
        super(DiagnosticReport_Image, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.comment is not None:
            self.comment.export(outfile, level, namespace_, name_='comment', pretty_print=pretty_print)
        if self.link is not None:
            self.link.export(outfile, level, namespace_, name_='link', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='DiagnosticReport.Image', mapping_=None):
        element = super(DiagnosticReport_Image, self).to_etree(parent_element, name_, mapping_)
        if self.comment is not None:
            comment_ = self.comment
            comment_.to_etree(element, name_='comment', mapping_=mapping_)
        if self.link is not None:
            link_ = self.link
            link_.to_etree(element, name_='link', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(DiagnosticReport_Image, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'comment':
            obj_ = string.factory()
            obj_.build(child_)
            self.comment = obj_
            obj_.original_tagname_ = 'comment'
        elif nodeName_ == 'link':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.link = obj_
            obj_.original_tagname_ = 'link'
        super(DiagnosticReport_Image, self).buildChildren(child_, node, nodeName_, True)
# end class DiagnosticReport_Image


class DiagnosticReportStatus(Element):
    """The status of the diagnostic report as a wholeIf the element is
    present, it must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(DiagnosticReportStatus, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if DiagnosticReportStatus.subclass:
            return DiagnosticReportStatus.subclass(*args_, **kwargs_)
        else:
            return DiagnosticReportStatus(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(DiagnosticReportStatus, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='DiagnosticReportStatus', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='DiagnosticReportStatus')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='DiagnosticReportStatus', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='DiagnosticReportStatus'):
        super(DiagnosticReportStatus, self).exportAttributes(outfile, level, already_processed, namespace_, name_='DiagnosticReportStatus')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='DiagnosticReportStatus', fromsubclass_=False, pretty_print=True):
        super(DiagnosticReportStatus, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='DiagnosticReportStatus', mapping_=None):
        element = super(DiagnosticReportStatus, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(DiagnosticReportStatus, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(DiagnosticReportStatus, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class DiagnosticReportStatus



#Copyright (c) 1999 Christian Scholz (ruebe@aachen.heimat.de)
#
#This library is free software; you can redistribute it and/or
#modify it under the terms of the GNU Library General Public
#License as published by the Free Software Foundation; either
#version 2 of the License, or (at your option) any later version.
#
#This library is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#Library General Public License for more details.
#
#You should have received a copy of the GNU Library General Public
#License along with this library; if not, write to the Free
#Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
#MA 02111-1307, USA



from .fileauth import DAVAuthHandler
import sys

class MySQLAuthHandler(DAVAuthHandler):
    """
    Provides authentication based on a mysql table
    """

    def get_userinfo(self,user,pw,command):
        """ authenticate user """

        # Commands that need write access
        nowrite=['OPTIONS','PROPFIND','GET']

        Mysql=self._config.MySQL
        DB=Mconn(Mysql.user,Mysql.passwd,Mysql.host,Mysql.port,Mysql.dbtable)
        if self.verbose:
            print(user,command, file=sys.stderr)

        qry="select * from %s.Users where User='%s' and Pass='%s'"%(Mysql.dbtable,user,pw)
        Auth=DB.execute(qry)

        if len(Auth) == 1:
            can_write=Auth[0][3]
            if not can_write and not command in nowrite:
                self._log('Authentication failed for user %s using command %s' %(user,command))
                return 0
            else:
                self._log('Successfully authenticated user %s writable=%s' % (user,can_write))
                return 1
        else:
            self._log('Authentication failed for user %s' % user)
            return 0

        self._log('Authentication failed for user %s' % user)
        return 0

